MIN_ROW = 0
MAX_ROW = 9
MIN_COL = 0
MAX_COL = 8
NUM_ROWS = MAX_ROW - MIN_ROW + 1
NUM_COLS = MAX_COL - MIN_COL + 1

CASTLE_MIN_COL = 3
CASTLE_MAX_COL = 5
CASTLE_TOP_MIN_ROW = 0
CASTLE_TOP_MAX_ROW = 2
CASTLE_BOT_MIN_ROW = 7
CASTLE_BOT_MAX_ROW = 9

HAN_ADVANTAGE = 1.5


import logging
import random
import time

from janggi import Camp
from janggi import Formation
from janggi import Location
from janggi import GameLog
from janggi import JanggiGame
from janggi import GameWindow
from janggi import ReplayViewer

logging.basicConfig()
logging.root.setLevel(logging.INFO)

if __name__ == '__main__':
    camp = Camp(random.choice([-1, 1]))
    cho_formation = Formation(random.randint(1, 4))
    han_formation = Formation(random.randint(1, 4))
    game = JanggiGame(camp, cho_formation, han_formation)
    print(f"cho: {game.cho_score} / han: {game.han_score}")
    print(game.board)

    # Test replay viewer
    moves = [(Location(3,0), Location(3,1)), (Location(6,8), Location(6,7)), (Location(0,0), Location(4,0))]
    game_log = GameLog(cho_formation, han_formation, camp, moves)
    game_log.generate_board_log()
    replay_viewer = ReplayViewer(game_log)
    replay_viewer.run()


from .game.game_log import GameLog
from .game.replay_viewer import ReplayViewer
from .proto import log_pb2

def replay(filepath: str):
    """
    Replay a game by parsing the log file at the given path.

    Args:
        filepath (str): Path of the proto-serialized log file.
    """
    log_file = open(filepath, "r")
    log_proto = log_pb2.Log()
    log_proto.ParseFromString(log_file.read())
    game_log = GameLog.from_proto(log_proto)
    replay_viewer = ReplayViewer(game_log)
    replay_viewer.run()


from typing import List, Tuple, Optional

from ..base.board import Board
from ..base.camp import Camp
from ..base.formation import Formation
from ..base.location import Location
from ..proto import log_pb2

class GameLog:
    """Simple class that represents list of moves made in a janggi game."""
    def __init__(self, cho_formation: Formation, han_formation: Formation, 
        bottom_camp: Camp, moves: Optional[List[Tuple[Location, Location]]] = []):
        self.cho_formation = cho_formation
        self.han_formation = han_formation
        self.bottom_camp = bottom_camp
        self.move_log = moves
        self.board_log = []
        self.index = 0

    def add_move(self, move: Tuple[Location ,Location]):
        """
        Add a single move to the move log

        Args:
            move (Tuple[Location ,Location]): _description_
        """
        self.move_log.append(move)

    def generate_board_log(self):
        board = Board.full_board_from_formations(
            self.cho_formation, self.han_formation, self.bottom_camp)
        self.board_log = [board.copy()]
        for origin, dest in self.move_log:
            board.move(origin, dest)
            self.board_log.append(board.copy())
        self.index = 0

    @classmethod
    def from_proto(cls, log_proto: log_pb2.Log):
        """Convert from proto Log message."""
        cho_formation = Formation.from_proto(log_proto.cho_formation)
        han_formation = Formation.from_proto(log_proto.han_formation)
        bottom_camp = Camp.from_proto(log_proto.bottom_camp)

        moves = []
        for move_proto in log_proto.moves:
            origin = Location.from_proto(move_proto.origin)
            dest = Location.from_proto(move_proto.dest)
            moves.append((origin, dest))
        return cls(cho_formation, han_formation, bottom_camp, moves)

    def to_proto(self):
        """Convert to proto Log message."""
        log_proto = log_pb2.Log()
        log_proto.cho_formation = self.cho_formation.to_proto()
        log_proto.han_formation = self.han_formation.to_proto()
        log_proto.bottom_camp = self.bottom_camp.to_proto()
        for origin, dest in self.move_log:
            move_proto = log_proto.moves.add()
            move_proto.origin = origin.to_proto()
            move_proto.dest = dest.to_proto()
        return log_proto

    def next(self) -> Board:
        """Used to access the next board when iterating through self.board_logs."""
        if self.index >= len(self.board_log) - 1:
            raise StopIteration
        self.index += 1
        board = self.board_log[self.index]
        return board

    def prev(self) -> Board:
        """Used to access the previous board when iterating through self.board_logs."""
        if self.index <= 0:
            raise StopIteration
        self.index -= 1
        board = self.board_log[self.index]
        return board
